package CarTypeRecognition::Classification;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;
use Storable;
use Chart::Graph::Gnuplot qw(&gnuplot);
use Time::Piece::ISO;

use CarTypeRecognition::SIFT;
use CarTypeRecognition::System;


our @ISA = qw(Exporter);
our $VERSION = '0.01';


sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	`mkdir -p $ENV{HOME}/.car-recognition/config`; 
	`mkdir -p $ENV{HOME}/.car-recognition/sift`;   

	my $class = ref($proto) || $proto;
	my $s   =CarTypeRecognition::System->new;

	my $self  = {
		'_SYSTEM'=>$s,
		'_SIFT'=>$props->{'-sift'},
		'_DBDIR' =>$props->{'-db-directory'},
	};

	bless ($self, $class);
	return $self;
}

sub get_system($)
{
	my $self = shift;
	return $self->{'_SYSTEM'};
}

sub get_sift($)
{
	my $self = shift;
	return $self->{'_SIFT'};
}


sub get_class_method_name($)
{
	
}

sub get_class_method_description($)
{

}

sub call_classification($)
{

}

# This does the classification process 
# i.e.: creates sifts for each car
sub classification_process($$)
{
	my $self=shift;
	my ($graphs)=@_;
	my $s=$self->get_system;
	my $sift=$self->get_sift;

	my @a;
	my @b;
	my $tr_n=@a=`find $self->{'_DBDIR'}/train-pics -type f`;
	my $ts_n=@b=`find $self->{'_DBDIR'}/test-pics -type f`;
	$self->{'ts_n'}=$ts_n;
	$self->{'tr_n'}=$tr_n;
	my ($car_train_dirs, undef)=$s->read_my_dir($self->{'_DBDIR'}."/train-pics");
	my ($car_test_dirs,  undef)=$s->read_my_dir($self->{'_DBDIR'}."/test-pics");

	my %car_classes_train=map { my (undef,$f)=$s->read_my_dir($_); $_=>$f } @$car_train_dirs;
	my %car_classes_test =map { my (undef,$f)=$s->read_my_dir($_); $_=>$f } @$car_test_dirs;
#	print "These are my car train classes: " . Dumper \%car_classes_train;
#	print "These are my car test classes: " . Dumper \%car_classes_test;

# Create SIFT for each car in the train set

	print "\n";
	my $i=0;
	map{
		$s->sync_fork_exec( $sift, 'create_SIFT',  $_  );
		$i++;
		printf("Stage 1: Creating SIFT for training cars ... %.2f%%\r",100*$i/$tr_n); 
	} map { @$_ } values %car_classes_train;

# Create SIFT for each car in the test set
	print "\n";
	$i=0;
	map{
		$s->sync_fork_exec( $sift, 'create_SIFT',  $_  );
		$i++;
		printf("Stage 2: Creating SIFT for testing cars ... %.2f%%\r",100*$i/$ts_n); 
	} map { @$_ } values %car_classes_test;


# This is the real classification procedure
#	print "car_classes_test=".Dumper \%car_classes_test;
#	print "car_classes_train=".Dumper \%car_classes_train;

	print "\n";
	my ($_classification, $_one_to_one, $_success)=$self->call_classification(\%car_classes_test, \%car_classes_train, $car_train_dirs);
	my %classification=%$_classification;
	my %one_to_one    =%$_one_to_one;

#	my %success=();
## Sort according to the error
#	for my $tc ( keys %classification ) {
#		my $tc_class=$s->get_car_class($tc);
#		print "$tc:\n";
#		my %class_confidence=%{$classification{$tc}};
#		my $c=0;
#		for my $k ( sort { $class_confidence{$a} <=> $class_confidence{$b} } keys %class_confidence ) {
#			$c++;
#			$success{$c}++ if( $tc_class eq basename $k ) ;
#		}
#	}
#
#	$success{$_}/=keys %classification for keys %success;
#
# printit

my %success=%$_success;
	my @x_column=sort { $a <=> $b } keys %success;
	my @y_column=();
	push @y_column, 100*$success{$_} for ( @x_column );
print Dumper \%success;
print Dumper \@x_column;
print Dumper \@y_column;


	push( @$graphs, [ 
			 {
			"title"=>$self->get_class_method_name(). ' ' .$sift->get_config() . ' Success: ' . $y_column[0] . '%',
			"style"=>"lines",
			"type" =>"columns",
			},
			\@x_column,
			\@y_column ] );

	print Dumper "GRAPHS=" . @$graphs;
			
	return (\%classification, \%one_to_one, $y_column[0]);			
}



# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
