package CarTypeRecognition::Classification::LDA;

use 5.008001;
use strict;
use warnings;
use File::Basename;
use Data::Dumper;
require Exporter;

require CarTypeRecognition::Classification;

our @ISA = qw(Exporter CarTypeRecognition::Classification);
our $VERSION = '0.01';

sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class=ref($proto) || $proto;
	my $self    =$class->SUPER::new($props);
	bless ($self, $class);
	return $self;
}

sub call_classification($)
{
	my ($self,$_car_classes_test, $_car_classes_train, $car_train_dirs)=@_;
	my %car_classes_test =%$_car_classes_test;
	my %car_classes_train=%$_car_classes_train;
	my $sift=$self->get_sift;
	my $s=$self->get_system;

#	print "TRAIN:".Dumper $_car_classes_train;
#	print "TEST:".Dumper $_car_classes_test;

	local *VECTORS_LEARN=*STDERR;
	local *VECTORS_TEST;

	my $vectors_learn="/tmp/vectors.learn-$$";
	open VECTORS_LEARN, ">$vectors_learn" or die "open($vectors_learn):$!\n";

# First, create vectors.learn file from train set:
	my $i=0; my $n=scalar(keys %car_classes_train);
	for (keys %car_classes_train){
		my $train_class=basename $_;
		print VECTORS_LEARN "# SIFT data for ``$train_class''\n";
		print VECTORS_LEARN "$train_class:\n";
		for my $train_car (@{$car_classes_train{$_}}){
			printf( "Stage 3: Train File for LDA ... %.2f%% complete\r",100*$i++/$n);
			my $train_sift=$sift->create_SIFT($train_car);
			for(@$train_sift){
				print VECTORS_LEARN "$_ " for @$_;
			}
			print VECTORS_LEARN "\n";
		}
	}
	close VECTORS_LEARN;

	print "\n";
# Next, create vectors.test from test cars
	my @all_cars=map {@$_} values %car_classes_test ;
	$n=@all_cars;
	my $vectors_test ="/tmp/vectors.test-$$";
	open VECTORS_TEST, ">$vectors_test"   or die "open($vectors_test):$!\n";
	$i=0;
	for my $test_car (@all_cars){
		printf( "Stage 4: Test File for LDA ... %.2f%% complete\r",100*$i++/$n);
		print VECTORS_TEST "# SIFT data for ``$test_car''\n";
		print VECTORS_TEST "$test_car:\n";
		my $test_sift=$sift->create_SIFT($test_car);
		for(@$test_sift){
			print VECTORS_TEST "$_ " for @$_;
		}
		print VECTORS_TEST "\n";
	}
	close VECTORS_TEST;
	
	print "\nCreating program for octave ...\n";
	my $output_file=(`octave/lda.pl $vectors_learn $vectors_test`)[0];
	chomp $output_file;
	my $cmd="cd octave && octave $output_file 2>&1|tee /var/tmp/classification.result";
	print "\nExecuting comand: $cmd\nPlease wait ... \n";
	my @output=`$cmd`;
	print Dumper \@output;
	
	print "Processing output ...\n";
# Process output
	my %success=();
	my %one_to_one=();
	my $report=0;
	$n=0;
	for(@output){
		chomp;
		$report=1,next if /^# classified/;
		next unless $report;
		next unless /^CLASSIFICATION:/;
		s/^CLASSIFICATION://g;
		my ($classified_as, $car)=split /,/,$_;
		$classified_as=~s/^\s*//g;
		$classified_as=~s/\s*$//g;
		$car  =~s/^\s*//g;
		$car  =~s/\s*$//g;
		my $real_class=$s->get_car_class($car);
		print STDERR "***DEBUG: classified as:|$classified_as| car=|$car| real_class=|$real_class|\n";
		$one_to_one{$car}=$classified_as;
		if( $real_class eq $classified_as ) {
			$success{'1'}++;
		}
		else {
			$success{'2'}++;
		}
	}
	print "Normalizing success vector\n";
	$n=scalar(keys %one_to_one);
	$success{$_}/=$n for keys %success;
	my %classification=();
	return (\%classification, \%one_to_one, \%success);
}

sub get_class_method_name($)
{
	my ($self)=@_;
	return 'LDA' ;
}

sub get_class_method_description($)
{
	return 'Here we classify using the LDA alghoritm.'; 
}


1;
