package CarTypeRecognition::Classification::kNN;

use 5.008001;
use strict;
use warnings;
use File::Basename;
use Data::Dumper;
require Exporter;

require CarTypeRecognition::Classification;

our @ISA = qw(Exporter CarTypeRecognition::Classification);
our $VERSION = '0.01';

sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class=ref($proto) || $proto;
	my $self    =$class->SUPER::new($props);
	$self->{'_K'}=$props->{'-k'} || 1;
	bless ($self, $class);
	return $self;
}

# k for k-NN classification
sub get_k($)
{
	return shift->{'_K'};
}

sub call_classification($)
{
	my ($self,$_car_classes_test, $_car_classes_train, $car_train_dirs)=@_;
	my %car_classes_test =%$_car_classes_test;
	my %car_classes_train=%$_car_classes_train;
	my $sift=$self->get_sift;
	my $s=$self->get_system;

#	print Dumper $_car_classes_train;
#	print Dumper $_car_classes_test;

# Do one_to_one matrix
	my %classification=();
	my %one_to_one=();
	my $i=0;	
	for my $test_car (map { @$_ } values %car_classes_test) {
#	print STDERR "Testing car : $test_car\n";

		# car class
		for  my $class ( @$car_train_dirs ) {
#		print "\tClass:" . basename $class. "\n";
			my $err_per_class=1000000.0; my $car_cnt=0;
			my @all_class_cars= map { @$_ } $car_classes_train{$class} ;
			for  my $train_car ( @all_class_cars ) {
				my $sift1=$sift->create_SIFT($train_car);
				my $sift2=$sift->create_SIFT($test_car);
				printf( "Stage 3: Classification process ... %.2f%%\r",100*$i++/($self->{'ts_n'}*$self->{'tr_n'}));
#print "train_car=$train_car test_car=$test_car\n";
#print "sift1=" . Dumper $sift1;
#print "sift2=" . Dumper $sift2;
				my $err   = $sift->compute_error( $sift1, $sift2 );
#print STDERR "\t\t$train_car: $err\n";
				$err_per_class=$err if $err<$err_per_class;
				$one_to_one{$test_car}=[] unless defined $one_to_one{$test_car};
				my $h= { "train-car"=>$train_car, "error"=>$err, "car-class" => $s->get_car_class($train_car), 'num-of-cars-in-class'=>scalar(@all_class_cars)};
				push @{$one_to_one{$test_car}}, $h;
				$car_cnt++;
			}
		#$err_per_class/=$car_cnt if $car_cnt>0;
#print "\tErr per class: $err_per_class\n";
			$classification{$test_car}{$class}=$err_per_class;
		}
	}


# Do k-NN classification from one_to_one matrix
	my %success=();
	my $k=$self->get_k();
	for my $test_car ( keys %one_to_one ) {
		my $test_car_class=$s->get_car_class($test_car);
		my @error_sorted_train_cars = sort { $a->{'error'} <=> $b->{'error'} } @{$one_to_one{$test_car}};
		my @k_best_cars = splice @error_sorted_train_cars, 0, $k;

#	print "TEST_CAR=$test_car\n";
#print "K_BEST_CARS=",Dumper \@k_best_cars;
		
		my %car_classes_count_in_k_best_cars=();
		$car_classes_count_in_k_best_cars{$_->{'car-class'}}+=1.0/$_->{'num-of-cars-in-class'} for(@k_best_cars);

#print "CAR_CLASSES_COUNT_IN_K_BEST_CARS=".Dumper \%car_classes_count_in_k_best_cars;
		
		my $test_car_recognized_class=(sort {
				$car_classes_count_in_k_best_cars{$b}<=>$car_classes_count_in_k_best_cars{$a}
			} keys %car_classes_count_in_k_best_cars)[0];
		if( $car_classes_count_in_k_best_cars{$test_car_recognized_class} == 1 ) {
			#fallback: take the closest
			$test_car_recognized_class=$error_sorted_train_cars[0];
		}
		if( $test_car_recognized_class eq $test_car_class ) {
			$success{'1'}++;
		}
		else {
			$success{'2'}++;
		}
	}
	$success{$_}/=keys %one_to_one for keys %success;

#	my %success=();
## Sort according to the error
#	for my $tc ( keys %classification ) {
#		my $tc_class=$s->get_car_class($tc);
##print "$tc:\n";
#		my %class_confidence=%{$classification{$tc}};
#		my $c=0;
#		for my $k ( sort { $class_confidence{$a} <=> $class_confidence{$b} } keys %class_confidence ) {
#			$c++;
#			$success{$c}++ if( $tc_class eq basename $k ) ;
#		}
#	}
#	
#	$success{$_}/=keys %classification for keys %success;

	return (\%classification, \%one_to_one, \%success);
}

sub get_class_method_name($)
{
	my ($self)=@_;
	return $self->get_k() . "-NN" ;
}

sub get_class_method_description($)
{
	return 'Here we classify using the k-NN alghoritm. It means we compute all distances and the nearest one is told to be the most probable one and the nearest one is told to be the most probable one.'
}


1;
