package CarTypeRecognition::Misc;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;

use CarTypeRecognition::System;
use CarTypeRecognition::Image;


our @ISA = qw(Exporter);
our $VERSION = '0.01';


sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	my $s   =CarTypeRecognition::System->new;
	my $im  =CarTypeRecognition::Image->new;

	my $self  = {
		'_SYSTEM'=>$s,
		'_IM_OBJ'=>$im,
		'_BASE_DIR'=>$props->{'-dir'} || './',
	};

	bless ($self, $class);
	return $self;
}

sub get_system($)
{
	my $self = shift;
	return $self->{'_SYSTEM'};
}

sub get_imobj($)
{
	my $self = shift;
	return $self->{'_IM_OBJ'};
}

sub get_base_dir($)
{
	my $self = shift;
	return $self->{'_BASE_DIR'};
}

sub get_save_name($$)
{
	my $self=shift;
	my ($car_class)=@_;
	return "$car_class-avg.png";
}

# Create average car from class
# returns filename and image object as a list ($f,$p)
sub average_car($$)
{
	my $self=shift;
	my (@cars)=@_;
	my $s       =$self->get_system();
	my $im      =$self->get_imobj();
	my $base_dir=$self->get_base_dir();

	# sum all cars
	my @imgs=(); my $car_cnt=0;
	my ($p, $ow, $oh);
	for my $car_filename (@cars) {
		($p, $ow, $oh) = $im->load_image( $car_filename ); 
		$car_cnt++;
		for my $x ( 0 .. $ow ) {
			for my $y ( 0 .. $oh ) {
				$imgs[$x][$y]+=$im->Iv($p,$x,$y);			
			}
		}
	}

#	print Dumper \@imgs;
	# average car
	for my $x ( 0 .. $ow ) {
		for my $y ( 0 .. $oh ) {
			$imgs[$x][$y]/=$car_cnt;
			my $c  =int($imgs[$x][$y]);
			my $col=sprintf("#%x%x%x", $c, $c, $c);
			
#			print "c=$col\n";
			$p->Set( "pixel[$x,$y]" =>$col );
		}
	}

	my $f=$self->get_save_name("avg.png");
	$p->Write($f);
	return ($f,$p);
}



# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
