package CarTypeRecognition::Report;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;
use Storable;
use Chart::Graph::Gnuplot qw(&gnuplot);

use CarTypeRecognition::System;

our @ISA = qw(Exporter);
our $VERSION = '0.01';


sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	my $s   =CarTypeRecognition::System->new;

	my $self  = {
		'_SYSTEM'=>$s,
		'_VIZ'=>$props->{'-viz-obj'},
		'_REPORT_ME'=>$props->{'-report-me'},
		'_REPORT_DIR'=>$props->{'-dir'} || 'report',
		'_SIFT'=>$props->{'-sift'},
		'_NAME'=>$props->{'-name'} || 'report',
#		'_GRAPHS'=>$props->{'-graphs'},
	};

	`mkdir -p $self->{'_REPORT_DIR'}`;
	bless ($self, $class);
	return $self;
}

sub get_vizobj($)
{
	my $self = shift;
	return $self->{'_VIZ'};
}

sub get_system($)
{
	my $self = shift;
	return $self->{'_SYSTEM'};
}

sub get_report($)
{
	my $self = shift;
	return $self->{'_REPORT_ME'};
}

sub get_sift($)
{
	my $self = shift;
	return $self->{'_SIFT'};
}

sub get_name($)
{
	my $self = shift;
	return $self->{'_NAME'};
}

sub get_graph($)
{
	my $self = shift;
	return $self->{'_GRAPHS'};
}

sub get_dir($)
{
	my $self = shift;
	return $self->{'_REPORT_DIR'};
}

sub report_text($)
{
	my $self=shift;
	my $s=$self->get_system;
	my $cls=$self->get_report;
	my $name=$self->get_name;
	my $report_dir=$self->get_dir();

	my $filename=$name . "-cross-" . $s->get_timestamp . ".txt";

	print "report_text(): The output HTML file location: $filename\n";
	local *FH;
	open FH, ">$report_dir/$filename" or die "open($filename): $!\n";


	my %classification=%$cls;
	my %success=();
# Sort according to the error
	for my $tc ( keys %classification ) {
		my $tc_class=($tc=~m#test-pics/(.*?)/#)[0];
		print FH "$tc:\n";
#print Dumper \%classification;
		my %class_confidence=%{$classification{$tc}};
		my $c=0;
		for my $k ( sort { $class_confidence{$a} <=> $class_confidence{$b} } keys %class_confidence ) {
			print FH "\t". basename $k . ":" . $class_confidence{$k} . "\n";
			$c++;
			$success{$c}++ if( $tc_class eq basename $k ) ;
		}
	}
	close FH;
}

# here one-to-one has to be passed as cls
sub report_cross($;$)
{
  my $self=shift;
	my $s=$self->get_system;
	my $cls=$self->get_report;
	my $sift=$self->get_sift;
	my $name=$self->get_name;
	my $k=shift;
	$k||=1;

	my $filename=$name . "-cross-" . $s->get_timestamp . ".html";
	print "report_cross(): The output HTML file location: $filename\n";
	my $report_dir=$self->get_dir();
	local *FH;
	open FH, ">$report_dir/$filename" or die "open($filename): $!\n";
	print FH "<html>\n<body>\n<h1>$filename</h1>\n<table>\n";

	my $v=$self->get_vizobj();
	die "no vizualization object!\n" unless defined $v;

	my %one_to_one=%$cls;
	for my $tc (keys %one_to_one) {
					my $tc_sfile=$v->visualize_SIFT($tc);
					my $tc_class=$s->get_car_class($tc);
					print FH "<tr>\n";
					my $classified_cars=$one_to_one{$tc};
#					print Dumper $l;
					my $i=0;
					my %class;
					for my $cc ( sort { $a->{'error'} <=> $b->{'error'} } @$classified_cars ) {
									$i++;
									my $n=$v->visualize_SIFT($cc->{'train-car'});
									my $c=$cc->{'car-class'};
									my $first=$c if $i==1;
									if( $i<=$k ) {
										$class{$c}++;
									}
									if($i==$k){
										my @L=sort {$class{$b}<=>$class{$a}} keys %class; 
										my $color="red";
										$color="#00ff00" if $L[0] eq $tc_class;
										print FH "<td bgcolor=\"$color\"><img src=\"$tc_sfile\" alt=\"$tc_class ($tc)\"title=\"$tc_class ($tc)\"></td>\n";
									}
									my $msg="(|$c|=". $cc->{'num-of-cars-in-class'} . ")";
									my $e=$cc->{'error'};
									print FH "  <td><img src=\"$n\" alt=\"$c err:=$e $msg\"title=\"$c err:$e $msg\"></td>\n";
					}
					print FH "</tr>\n";
	}

	print FH "</body>\n</html>\n";

	close FH;


}

sub report_gnuplot($$)
{
  my $self=shift;
	my $s=$self->get_system;
	my $cls=$self->get_report;
	my $sift=$self->get_sift;
	my ($graphs) = @_;
	print STDERR "Graphs=" . Dumper $graphs;
	
	my $opts={
			'title'        =>$self->get_name(),
			'x-axis label' =>'Classification Rank',
			'y-axis label' =>'Success [%]',
			'output type'  =>'gif',
			'output file'  =>'/tmp/ROC-'. $self->get_name() . '-' . $s->get_timestamp() .'.gif',
			};


	gnuplot( $opts, @$graphs );
}



# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
