package CarTypeRecognition::SIFT;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;

use CarTypeRecognition::Image;
use CarTypeRecognition::System;

our @ISA = qw(Exporter);
our $VERSION = '0.01';

my $PI=3.141592653590;

sub get_sift_name($)
{
	return "SIFT-simple";
}

sub get_sift_description($)
{
}

sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
#	print "SIFT: class=$class\n";
	system("mkdir -p " . $props->{'-sift-dir'} );

	my $im=CarTypeRecognition::Image->new || die "Cannot create Image object!\n";
	my $s =CarTypeRecognition::System->new || die "Cannot create System object!\n";
	
	my $self  = {
		'_GRAD_BINS'  => $props->{'-grad-bins'} || 9,
		'_BOXES_HORIZ'=>$props->{'-boxes-horiz'} || 8,
		'_BOXES_VERT' =>$props->{'-boxes-vert'} || 4,
		'_BASE_DIR'   => $props->{'-sift-dir'},
		'_ERRFUNC'    => $props->{'-error-func'},
		'_MY_NAME'    => $class,
		'_IM_OBJ'     => $im,
		'_SYS_OBJ'    => $s,
		'overlap' => $props->{'-overlap'},
	};

	bless ($self, $class);

#	print "***my config" . $self->get_config();
	return $self;
}

sub get_overlap($)
{
	my($self)=@_;
	my $overlap='overlap';
	$overlap='nooverlap' unless $self->{'overlap'};
	return $overlap;
}

# Gets current SIFT configuration
sub get_config($)
{
	my $self=shift;

	my $s=join("x", ( 
				$self->{'_BOXES_HORIZ'},
				$self->{'_BOXES_VERT'},
				$self->{'_GRAD_BINS'}
				));
	return $self->get_sift_name()."-$s-" . $self->get_overlap();
}


# Get image object
sub get_imobj($)
{
				my $self=shift;
				return $self->{'_IM_OBJ'};
}

sub get_systemobj($)
{
				my $self=shift;
				return $self->{'_SYS_OBJ'};
}

sub get_errfunc($)
{
	return shift->{'_ERRFUNC'};
}


sub get_sign_coords($)
{
	return ( 95, 55, 176, 76 );
}


sub inside_sign($$$)
{
	my ($self,$x,$y)=@_;
	my ($sx1,$sy1,$sx2,$sy2)=$self->get_sign_coords();
	return 1 if ($x>$sx1 && $x<$sx2 && $y>$sy1 && $y<$sy2);
	return 0;
}

sub get_coords_filename($$)
{
	my $self=shift;
	my ($file)=@_;

	die "Usage: get_coords_filename(\$filename)\n" unless $file;
	my $bv=$self->{'_BOXES_VERT'};
	my $bh=$self->{'_BOXES_HORIZ'};
	my $gb=$self->{'_GRAD_BINS'};
	my $name=$self->{'_MY_NAME'};
	my $base_dir=$self->{'_BASE_DIR'};

	my $f="$base_dir/" . basename $file ."-$name-$bv-$bh-$gb-".$self->get_overlap()."-coords";
#	print "***F=$f\n";
	return $f;
}

sub get_sift_filename($$)
{
	my $self=shift;
	my ($file)=@_;
	die "Usage: get_sift_filename(\$filename)\n" unless $file;
	my $bv=$self->{'_BOXES_VERT'};
	my $bh=$self->{'_BOXES_HORIZ'};
	my $gb=$self->{'_GRAD_BINS'};
	my $name=$self->{'_MY_NAME'};
	my $base_dir=$self->{'_BASE_DIR'};

	return "$base_dir/" . basename $file ."-$name-$bv-$bh-$gb-".$self->get_overlap();
}


# Gets array of coordinates from file
sub get_coords($$)
{
	my $self=shift;
	my ($file)=@_;
	my $s=$self->get_systemobj();
	my $lookup_filename_coords=$self->get_coords_filename($file);

	my $c=$s->load_var( $lookup_filename_coords );
	die "get_coords(): Unable to load coords from file $lookup_filename_coords\n" unless $c;
	return $c;
}

# Create SIFT
sub create_SIFT($$)
{
	my $self=shift;
	my ($file) = @_;
	my $base_dir=$self->{'_BASE_DIR'};

	my $s=$self->get_systemobj();

#	print "pid $$ : create_SIFT(): file=$file\n";
	my $bv=$self->{'_BOXES_VERT'};
	my $bh=$self->{'_BOXES_HORIZ'};
	my $gb=$self->{'_GRAD_BINS'};
	my $name=$self->{'_MY_NAME'};


	my $lookup_filename_sift  =$self->get_sift_filename  ($file);
	my $lookup_filename_coords=$self->get_coords_filename($file);

#	print "lookup_filename_sift=$lookup_filename_sift\n";
	my $SIFT=[];
	my $SIFT_Coords=[];
	my $sift_ref = $s->load_var( $lookup_filename_sift );
	return $sift_ref if $sift_ref;

	my $im=$self->get_imobj();
	my ($p, $ow, $oh) = $im->load_image( $file );

	if( $ow>256 ) { 
		# scale image
		$oh/=($ow/256);
		$ow=256;
		$p->Scale("${ow}x${oh}");
		print "Image Scaled down to ${ow}x${oh}\n";
	}

	die "undefined width\n" unless defined $ow;
	die "undefined height\n" unless defined $oh;

	
	# Box size
	my $stw = (($ow>60) ? ($ow/$bh) : ($ow)) ;
	my $sth = (($oh>60) ? ($oh/$bv) : ($oh)) ;
		
	my ( $sx1, $sy1, $sx2, $sy2 ) = $self->get_sign_coords();

	for ( my $x=1; $x<$ow; $x+=$stw ) {
		for ( my $y=1; $y<$oh; $y+=$sth ) {
			my @h=$self->directions_hist( $p, $x, $y, $stw, $sth );
			push( @$SIFT, \@h );
			push( @$SIFT_Coords, [ $x, $y, $stw, $sth ] );
		}
	}

	if( $self->{'overlap'} ) {
		for ( my $x=$stw/2; $x<$ow-$stw/2; $x+=$stw ) {
			for ( my $y=$sth/2; $y<$oh-$sth/2; $y+=$sth ) {
				my @h=$self->directions_hist( $p, $x, $y, $stw, $sth );
				push( @$SIFT, \@h );
				push( @$SIFT_Coords, [ $x, $y, $stw, $sth ] );
			}
		}
	}

	print "create_SIFT(): \$SIFT=". Dumper $SIFT;
#	print "create_SIFT(): \$SIFT_Coords=". Dumper $SIFT_Coords;
	undef $p;

	$s->save_var( $SIFT       , $lookup_filename_sift );
	$s->save_var( $SIFT_Coords, $lookup_filename_coords );
	
	return $SIFT;
}

# Creates a list of directions in a box
sub directions_hist($$$$$$)
{
	my $self=shift;
	my $GRAD_BINS=$self->{'_GRAD_BINS'};
	my ( $p, $sx,$sy, $w, $h ) = @_;

	
#$p->Convolve( [ 1, 1, 1, 1, -8, 1, 1, 1, 1 ] );


	my @h=();
	for(my $i=0; $i<$GRAD_BINS; $i++) { $h[$i] = 0 }

	my ($sx1, $sy1, $sx2, $sy2 )=$self->get_sign_coords();
	my $grad_sum=0;
	for my $y ( $sy .. $sy+$h-1 ) {
		for my $x ( $sx .. $sx+$w-1 ) {
			# skip sign
			next if $self->inside_sign($x,$y);
# 95,55
# 176,76			

			my $a = int($self->compute_derivation( $p, $x, $y ));

			$h[ int($a/(180/$GRAD_BINS)) ] ||= 0;
			$h[ int($a/(180/$GRAD_BINS)) ] ++;
			$grad_sum++;

#			print "h=$h[int($a/$GRAD_BINS)]\n";


		}
	}

	
	$grad_sum=1 unless $grad_sum;
# FIXME: different results with and without normalization?!
	$_/=$grad_sum for @h;
#print "x=$sx y=$sy grad_sum=$grad_sum\n";
	return @h;
}

# Computes error from SIFT 
sub compute_error($$$)
{
	my ($self,$s1,$s2)=@_;
	return $self->get_errfunc()->get_error($_[1],$_[2]);
}



# Loads image
# Computes derivation via Sobel
sub compute_derivation($$$$)
{
	my $s=shift;
	my ($p,$x,$y)=@_;

	my $im=$s->get_imobj();

	my $dy =  $im->Iv( $p, $x-1, $y-1 ) + 2*$im->Iv( $p, $x, $y-1 ) + $im->Iv( $p, $x+1, $y-1 ) -
		    ( $im->Iv( $p, $x-1, $y+1 ) + 2*$im->Iv( $p, $x, $y+1 ) + $im->Iv( $p, $x+1, $y+1 ) );

	my $dx =  $im->Iv( $p, $x+1, $y-1 ) + 2*$im->Iv( $p, $x+1, $y ) + $im->Iv( $p, $x+1, $y+1 ) -
		    ( $im->Iv( $p, $x-1, $y-1 ) + 2*$im->Iv( $p, $x-1, $y ) + $im->Iv( $p, $x-1, $y+1 ) );

	my $a=atan2 $dy, $dx;

	#print "dx=$dx dy=$dy a=$a";
	$a*=180/$PI;
	$a+=180;
	$a%=180;
	#print "($a)\n";

	return $a;
}





# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
