package CarTypeRecognition::SIFT::NormalizeGradBlur;

# This package uses normalized gradient to simulate the probability
# of certain angles:
#          /\     f(x), sum(f(x))=1, -infty, +infty
#         /  \--/\
#    ..../        \....    
use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;
use Storable;

use CarTypeRecognition::SIFT;
use CarTypeRecognition::Image;

our @ISA = qw(Exporter CarTypeRecognition::SIFT);
our $VERSION = '0.01';

sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	print "NormalizeGrad: class=$class\n";
	
	my $self = $class->SUPER::new( $props );

	bless ($self, $class);

	print "***my config" . $self->get_config();
	return $self;
}

sub get_sift_name($)
{
	return "SIFT with norm-gradients-blur";
}

sub get_sift_description($)
{
	return 'This package normalizes gradients to simulate the probability of certain angles. In fact it makes a distribution'.
		'function from SIFT computed gradients.'
}


# Creates a list of directions in a box
sub directions_hist($$$$$$)
{
	my $self=shift;
	my $GRAD_BINS=$self->{'_GRAD_BINS'};
	my ( $p, $sx,$sy, $w, $h ) = @_;
	my $cx=$sx+$w/2;
	my $cy=$sy+$h/2;

	
#$p->Convolve( [ 1, 1, 1, 1, -8, 1, 1, 1, 1 ] );


	my @h=();
	for(my $i=0; $i<$GRAD_BINS; $i++) { $h[$i] = 0 }

	
	my $grad_sum=0; 
	for my $y ( $sy .. $sy+$h-1 ) {
		for my $x ( $sx .. $sx+$w-1 ) {
			# skip sign
			next if $self->inside_sign($x,$y);
# 95,55
# 176,76			

			my $a   =int($self->compute_derivation( $p, $x, $y ));
			my $g   =    $self->compute_grad      ( $p, $x, $y );
			my $dist=sqrt(($cx-$x)*($cx-$x) + ($cy-$y)*($cy-$y)); 
			$dist++ if !$dist;
			$g/=$dist;
		
			#print "a=$a g=$g\n";
			$h[ int($a/(180/$GRAD_BINS)) ] ||= 0;
			$h[ int($a/(180/$GRAD_BINS)) ] += $g;
			$grad_sum+=$g;
			#print Dumper \@h;
			

#			print "h=$h[int($a/$GRAD_BINS)]\n";
		}
	}
	$grad_sum=1 unless $grad_sum;
	$_/=$grad_sum for @h;
	return @h;
}


sub compute_grad($$$$)
{
	my $s=shift;
	my ($p,$x,$y)=@_;

	my $im=$s->get_imobj();
	my $dy =  $im->Iv( $p, $x-1, $y-1 ) + 2*$im->Iv( $p, $x, $y-1 ) + $im->Iv( $p, $x+1, $y-1 ) -
		    ( $im->Iv( $p, $x-1, $y+1 ) + 2*$im->Iv( $p, $x, $y+1 ) + $im->Iv( $p, $x+1, $y+1 ) );

	my $dx =  $im->Iv( $p, $x+1, $y-1 ) + 2*$im->Iv( $p, $x+1, $y ) + $im->Iv( $p, $x+1, $y+1 ) -
		    ( $im->Iv( $p, $x-1, $y-1 ) + 2*$im->Iv( $p, $x-1, $y ) + $im->Iv( $p, $x-1, $y+1 ) );

	$dx/=256;
	$dy/=256;
	return sqrt($dx*$dx+$dy*$dy);
}

# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
