package CarTypeRecognition::Stat;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Data::Dumper;
use File::Basename;
use Storable;
use PDL;
use PDL::Matrix;
use PDL::MatrixOps;

use CarTypeRecognition::System;


our @ISA = qw(Exporter);
our $VERSION = '0.01';


sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	my $s   =CarTypeRecognition::System->new || die "Cannot create System object\n";

	my $self  = {
		'_SYSTEM'=>$s,
	};
	bless ($self, $class);
	return $self;
}

sub get_system($)
{
	my $self = shift;
	return $self->{'_SYSTEM'};
}

sub add_v($$$)
{
	my( $self, $a, $b ) = @_;

	for my $i ( 0..$#$a ) {
			$a->[$i]+=$b->[$i];
	}

	return wantarray ? @$a: $a;
}

sub sub_v($$$)
{
	my( $self, $a, $b ) = @_;

	for my $i ( 0..$#$a ) {
			$a->[$i]-=$b->[$i];
	}

	return wantarray ? @$a: $a;
}


sub zero_v($$$)
{
	my( $self, $a, $b ) = @_;

	for my $i ( 0..$#$b ) {
			$a->[$i]=0;
	}

	return wantarray ? @$a: $a;
}

sub quad_v($$)
{
	my( $self, $a ) = @_;

	for my $i ( 0..$#$a ) {
			$a->[$i]*=$a->[$i];
	}

	return wantarray ? @$a: $a;
}

sub abs_v($$)
{
	my( $self, $a ) = @_;
	
	my $d=0;
	for my $i ( 0..$#$a ) {
			$d+=$a->[$i]*$a->[$i];
	}

	return $d;
}


sub make_matrices($@)
{
	my ($self,@class_data)=@_;
	my @m=();
	push @m,PDL::Matrix->pdl($_) for (@class_data);
	return @m;
}

sub mean($@)
{
	my ($self,@class_data)=@_;
	@class_data=$self->make_matrices(@class_data) if ref $class_data[0] ne 'PDL::Matrix' ;
	my $n=@class_data; # number of vectors in data vector
	my $m=PDL::Matrix->zeroes(1,$class_data[0]->ndims);
	$m+=$_ for (@class_data);
	$m/=$n;
	return $m;
}

sub S_i($@)
{
	my ($self,@class_data)=@_;
	@class_data=$self->make_matrices(@class_data) if ref $class_data[0] ne 'PDL::Matrix' ;
	
	my $s=PDL::Matrix->zeroes($class_data[0]->ndims,$class_data[0]->ndims);
	my $m=$self->mean(@class_data);
	for my $x ( @class_data ) {
		my $n_i=@class_data;
		my $d=transpose($x-$m)*($x-$m);
		$s+=$d;	
#		print "x=$x m=$m S=$s D=$d\n";
	}
	return $s;
}

# Scatter within class (uses S_i)
# pass array of array refs:
#  ( [ .... ] - class1
#    [ .... ] - class2 
#     ...   etc.
sub S_w($@)
{
	my ($self,@class_data)=@_;
	for my $i ( 0..$#class_data) {
		if( ref $class_data[$i]->[0] ne 'PDL::Matrix' ) {
			my @_c=$self->make_matrices(@{$class_data[$i]});
			$class_data[$i]=\@_c;
		}
	}

	my $s_w=PDL::Matrix->zeroes($class_data[0]->[0]->ndims,$class_data[0]->[0]->ndims);
	for my $c (@class_data) {
		my $d=$self->S_i(@$c);	
		$s_w+=$d; 	
		print "d=$d s_w=$s_w\n";
	}
	return $s_w;
}

# Total mean
sub total_mean($@)
{
	my ($self,@class_data)=@_;
	for my $i ( 0..$#class_data) {
		if( ref $class_data[$i]->[0] ne 'PDL::Matrix' ) {
			my @_c=$self->make_matrices(@{$class_data[$i]});
			$class_data[$i]=\@_c;
		}
	}
	my $n=0;
	my $m=PDL::Matrix->zeroes(1,$class_data[0]->[0]->ndims);
	for my $c (@class_data) {
		my $m_i=$self->mean(@$c);
		my $n_i=@$c;
		$m+=$n_i*$m_i;
		$n+=$n_i;	
		print "n_i=$n_i m_i=$m_i\n";
	}
	print "n=$n\n";
	$m/=$n;
	return $m;
}

sub S_b($@)
{
	my ($self,@class_data)=@_;
	for my $i ( 0..$#class_data) {
		if( ref $class_data[$i]->[0] ne 'PDL::Matrix' ) {
			my @_c=$self->make_matrices(@{$class_data[$i]});
			$class_data[$i]=\@_c;
		}
	}

	my $m=$self->total_mean(@class_data);

	my $s_b=PDL::Matrix->zeroes($class_data[0]->[0]->ndims,$class_data[0]->[0]->ndims);
	for my $c (@class_data) {
		my $m_i=$self->mean(@$c);
		my $n_i=@$c;
		my $d=$n_i*transpose($m_i-$m)*($m_i-$m);
		#my $d=transpose($m_i-$m)*($m_i-$m);
		$s_b+=$d; 	
		print "d=$d s_b=$s_b\n";
	}
	return $s_b;
}

sub gen_eigens($@)
{
	my ($self,@class_data)=@_;
	for my $i ( 0..$#class_data) {
		if( ref $class_data[$i]->[0] ne 'PDL::Matrix' ) {
			my @_c=$self->make_matrices(@{$class_data[$i]});
			$class_data[$i]=\@_c;
		}
	}

	my $Sw=$self->S_w( @class_data );
	my $Sb=$self->S_b( @class_data );
	my $A =inv($Sw) x $Sb;
	my ( $u, $s, $v ) = PDL::MatrixOps::svd(transpose($A));

	print "gen_eigens(): Sw=$Sw\n";
	print "gen_eigens(): Sb=$Sb\n";
	print "gen_eigens(): A=$A\n";
	print "gen_eigens(): u=$u\n";
	print "gen_eigens(): s=$s\n";
	print "gen_eigens(): v=$v\n";
	$v=-$v;
	print "gen_eigens(): v=$v\n";
	
	print $v;
	return $v;	
}

sub lda_classify($$$;$)
{
	my ($self,$_x,$_class_data, $dim)=@_;
	my @class_data=@$_class_data;
	my @x         =@$_x;
	@x            =$self->make_matrices(@x)  if ref $x[0] ne 'PDL::Matrix' ;
	for my $i ( 0..$#class_data) {
		if( ref $class_data[$i]->[0] ne 'PDL::Matrix' ) {
			my @_c=$self->make_matrices(@{$class_data[$i]});
			$class_data[$i]=\@_c;
		}
	}

	my $cn=@$_class_data;
	print "Class number : cn=$cn\n";
	die "lda_classify: What you want to classify?!\n" if $cn<=1;
	$dim||=$cn-1;
	my $dim_index=$dim-1;
	

	my $m=$self->total_mean(@class_data);
	my $W_p=$self->gen_eigens(@class_data);
	my $W=$W_p->slice("0:$dim_index,:");

	print "***lda_classify: W_p=$W_p\n";
	print "***lda_classify: W=$W\n";

	my $n=$W->ndims;
	print "lda_classify: x=$x[0]\n";
	print "lda_classify: W=$W";
	# Compute the projected test vector
	my $x_p =  $x[0] x transpose $W;
	print "lda_classify: x_p=$x_p\n";
	for my $c ( @class_data ) {
		my $m_i = $self->mean(@$c);	
# Project the mean into the computed space
		my $m_ip= $m_i x transpose $W;
		my $d   = $x_p-$m_ip;
		print "lda_classify: m_ip=$m_ip\n";
		print "lda_classify: d=$d\n";
		$d*=$d;
		print "DIMENS:", $d->ndims;
		print "lda_classify: d=$d\n";
	}
}




1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Data::Dumper;
  use CarTypeRecognition::Stat; 

  $s=CarTypeRecognition::Stat->new; 

  my @res=$s->average([1,2,3],[1,1,1],[3,3,3]); 
  print Dumper \@res;

  -----------------------------------------------
  prints:
 
$VAR1 = [
          '1.66666666666667',
          '2',
          '2.33333333333333'
        ];

  -----------------------------------------------
  my @res=$s->variance([1,2,3],[1,1,1],[3,3,3]); 

# Here is how we can compute the size of the variance vector:
 my $size=$s->abs_v(\@{$s->variance(@a)})

=head1 DESCRIPTION
	average
		- takes an array of array refs and computes average

	variance
		- takes an array of array refs and computes variance

	abs_v
		- absolute value of vector

	add_v
		- add two vectors

	sub_v 
		- substract two vectors	

	

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
