package CarTypeRecognition::System;

use 5.008001;
use strict;
use warnings;
use POSIX ":sys_wait_h";
use Time::Piece::ISO;
use Storable;

require Exporter;

our @ISA = qw(Exporter);

our $VERSION = '0.01';

sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	
	my $self  = {
		'-base-dir' => $props->{'-base-dir'} || './',
	};

	bless ($self, $class);
	return $self;
}

sub get_basedir($)
{
	return shift->{'-base-dir'};
}

sub read_my_dir($$)
{
	my $self=shift;
	my ($dir)=@_;
	my @odirs=();
	my @ofiles=();
	local *FD;
	opendir  FD, $dir or die "opendir($dir): $!\n";
	my @files=readdir(FD);
	for my $f ( @files ) {
		next if ($f=~/^\./);
		push @odirs,  $f if -d $dir."/".$f;
		push @ofiles, $f if -f $dir."/".$f;
	}
	closedir FD;
	$_=$dir."/".$_ for @odirs;
	$_=$dir."/".$_ for @ofiles;
	return (\@odirs, \@ofiles);
}

sub save_var($$$)
{
	my $self=shift;
	my ( $var, $lookup_filename_sift )=@_;

	system("rm -f /tmp/sift");
	die "rm(1): cannot delete /tmp/sift\n" if $?;
	store $var, "/tmp/sift";
	system("mv -f /tmp/sift \"$lookup_filename_sift\""); # or die "Cannot run mv(1)\n";
	die "mv(1) of /tmp/sift to $lookup_filename_sift was not successful\n" if $?; 
}

sub load_var($$)
{
	my $self=shift;
	my ( $lookup_filename_sift )=@_;
	if( -f $lookup_filename_sift ) {
		my $sift_ref = retrieve($lookup_filename_sift);	
#print "\t(retrieved from file)\n";
		return $sift_ref;
	}
	else {
		return undef;
	}
}



# Workaround for perl and/or imagemagick
# leak of memory per image 10-20MB
# TODO: recompile perl and imagemagick
sub sync_fork_exec($$$$;$)
{
	my $self=shift;
	my ($o,$f,$arg, $timeout)=@_;
#	print "sync_forked_exec(): arg=$arg\n";
	my $pid=fork();
	die "Cannot fork: fork() $!\n" if $pid==-1;
	if ( $pid ) {
		my $t=0;			
		for (;;) {
			my $kid=waitpid(-1,WNOHANG);
			select( undef, undef, undef, 0.1 );
			last if $kid==$pid;
			last if $timeout && $t>$timeout;
			$t++;
		}
#		print "sync_forked_exec(): child exited\n";
	} else {
		eval $o->$f($arg);
		exit;
	}
}

sub get_nums_for_strings($@)
{
	my ($self, @str_array)=@_;
	my $c=0;
	my %strh=();
	for my $s (@str_array){
		if ( !$strh{$s} ) {
			$strh{$s}=++$c;
		}
	}

	return \%strh;
		
}

# Get class name for car
sub get_car_class($$)
{
	my $self=shift;
	my ($tc)=@_;
	my $tc_class=($tc=~m#test-pics/(.*?)/#)[0];
	return  $tc_class if $tc_class;
	$tc_class=($tc=~m#train-pics/(.*?)/#)[0];
	return  $tc_class if $tc_class;
	return "unknown-cannot-determine";
}

sub get_cars_for_class($$$)
{
	my ($self, $class_name)=@_;

	my $db_dir=$self->get_basedir();
	my (undef, $files)=$self->read_my_dir("$db_dir/$class_name/");
	return @$files;
}

# Gets timestamp according to the ISO
sub get_timestamp($)
{
	return Time::Piece::ISO::localtime() . "+00:00",
}


1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::System - Perl extension for CarTypeRecognition

=head1 SYNOPSIS

  use CarTypeRecognition::System;

	my $s=CarTypeRecognition::System->new;
	( $dirs, $files ) = $s->read_my_dir( $my_dir );

	$s->sync_forked_exec( $obj, 'func_name', $arg );

	my $time=$s->get_timestamp();


=head1 DESCRIPTION

=head3 read_my_dir()

reads files and directories in $my_dir; returns references to lists of directories 
and files located under $my_dir

=head3 sync_forked_exec()

executes func_name in parallel to the parent process. The parent process
waits untill your function exits

=head3 get_timestamp()

Returns string holding the current time according to the IS 862				

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Michal Conos, <mikc@vellum.cz>
Faculty of Electrical Engineering
Czech Technical University

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
