#!/usr/bin/perl

use strict;
use warnings;

use YAML;
use Data::Dumper;
use Getopt::Long;
use File::Basename;

sub nn($$);
sub compute_success(%);

my $test_dbs;
my $input_file;
my $help=<<EOH;
$0 --file|f <filename> --testing-databases|d <dir1,dir2,...>
EOH
my %Options=(
		'--testing-databases|d=s'   => \$test_dbs,
		'--file|f=s' => \$input_file,
		);
die $help unless @ARGV;
GetOptions( %Options ) or die "GetOptions failure: $!"; 

my $d=YAML::LoadFile($input_file);

my @test_databases=split/,/,$test_dbs;

my $learn_set=$d->[0]{'data'};
#print Dumper $learn_set;

for(@test_databases){
	for my $e (@$d){
		if( $e->{'name'} eq $_ ){
			print "TEST of $_\n";
			my $test_set=$e->{'data'};	
#print Dumper $test_set;
			my %classification=nn($learn_set, $test_set);
			print STDERR "CLS=",Dumper \%classification;
			my $success=compute_success(%classification);
			print STDERR "SUCCESS=",Dumper $success;
		}
	}
}

sub get_car_class($)
{
	my ($i)=@_;
	my $d=File::Basename::dirname $i;
	$d=~s/\/$//g;
	my $class=File::Basename::basename $d;
	return $class if $class;
	die "Cannot determine class for $i!\n"; 
}


sub compute_success(%)
{
my(%classification)=@_;
my %success=();	
	for my $t (keys %classification){
		my $c1=get_car_class $t;
		my $c2=get_car_class $classification{$t};
		if( $c1 eq $c2 ) {
			$success{'positive'}++;
		}
		else {
			$success{'negative'}++;
		}
	}
	return \%success;
}

sub error($$)
{
	my ($v1, $v2)=@_;
	my @v1=split(/ /,$v1);
	my @v2=split(/ /,$v2);

	my $err=0;
	my $n=@v1;
	$n--;
	for my $i ( 0 .. $n ){
		my $dx=$v1[$i]-$v2[$i];
		$err+=$dx*$dx;	
	}

#print STDERR Dumper \@v1;
#print STDERR Dumper \@v2;
#print STDERR "err=$err\n";
	return $err;
}

sub nn($$)
{
	my($learn_set, $test_set)=@_;

	my %classification=();
	my $best;
	
	for my $t (keys %$test_set){
		my $min_e=1000000;
		for my $l (keys %$learn_set){
			my $vl=$learn_set->{$l};
			my $vt=$test_set->{$t};
			my $e=error($vl, $vt);
			if( $e<$min_e ) {
				$min_e=$e;
				$best=$l;
			}
		}
		$classification{$t}=$best;
	}
	return %classification;
}
