#!/usr/bin/perl

use strict;
use warnings;

use YAML;
use Data::Dumper;
use Getopt::Long;
use File::Basename;

sub get_car_class($)
{
	my ($i)=@_;
	my $d=File::Basename::dirname $i;
	$d=~s/\/$//g;
	my $class=File::Basename::basename $d;
	return $class if $class;
	die "Cannot determine class for $i!\n"; 
}

sub make_lda($)
{
my ($e)=@_;	
my $TOP=0;
my %classes;
my $y_vec="";
print FH "X=[\n";
my $db     =$e->{'data'};
my $db_name=$e->{'name'};
for my $d (keys %$db){
	my $c=get_car_class $d;
	my $v=$db->{$d}; # vector
	print FH $v,"\n";	
	unless ($classes{$c}) {
		$TOP++;
		$classes{$c}=$TOP;
	}
	$y_vec.="$classes{$c} ";
}
print FH "];\n";
my $out=<<EOO;
y=[
	$y_vec
];
D.X=X';
D.y=y;
model=lda(D,max(y)-1);

EOO
print FH $out;
#print Dumper \%classes;

}

sub transform_lda($)
{
my ($e)=@_;	
my $db     =$e->{'data'};
my $db_name=$e->{'name'};
	print FH "printf(\"- name: $db_name\\n\");\n"; # print - name
	print FH "printf(\"  data:\\n\");\n";             # print   data
	for my $d (keys %$db){
		my $v=$db->{$d}; # vector
		print FH "printf(\"    $d: \");";
		print FH "printf(\"%f \", [$v]*model.W);\n";             # print img: vector
		print FH "printf(\"\\n\");\n";             # print   data
	}
	print FH "\n";
}

my $input_file;
my $output_file;
my $help=<<EOH;
$0 --input-file|i <filename> --output-file|o <filename>
EOH
my %Options=(
		'--input-file|f=s' => \$input_file,
		'--output-file|f=s' => \$output_file,
		);
die $help unless @ARGV;
GetOptions( %Options ) or die "GetOptions failure: $!"; 

open FH, "> $output_file" or die "open($output_file):$!\n";

my $d=YAML::LoadFile($input_file);
#print Dumper $d;
make_lda($d->[0]);

for(@$d){
	transform_lda($_);
}


close FH;
