#!/usr/bin/perl

use warnings;
use strict;
use Data::Dumper;
use File::Basename;

use Getopt::Long;

my $test_ratio=0.5;
my $opt_help=0;
my $sift_type='normalized-grad-blur';
my $db_dir;
my $output_db_dir;

my $help=<<EOH;
$0 [options] <directory>
	--test-ratio|r          : test : train cars ratio [default is 50%]
	--db-dir|d              : Car source database directory
	--output-db-dir|o       : Car output database directory
	--help                  : This help
EOH

my %Options=(
			'--db-dir|d=s' => \$db_dir,
			'--output-db-dir|o=s' => \$output_db_dir,
			'--help' => \$opt_help,
			'--test-ratio|r=f' => \$test_ratio,
			);

sub validate_config()
{
}

sub print_config()
{
	print "Car database: $db_dir\n";
	print "Test:Train ratio: $test_ratio\n";
}

die $help if @ARGV==0;
GetOptions( %Options ) or die "GetOptions failure: $!"; 
die $help if $opt_help;
die $help unless $db_dir;
die $help unless $output_db_dir;
validate_config();
print_config();


my @files=`find $db_dir -type f -print`;
my %c=();

for(@files){
	chomp;
	my @_dummy=split /\//, dirname $_;
	my $class=pop @_dummy;
	$c{$class}=[] unless $c{$class};
	push @{$c{$class}}, $_;
}

my $total=0; my $n;
print "$_:", $n=scalar(@{$c{$_}}), " total=", $total+=$n,"\n" for(keys %c);

system "rm -rf $output_db_dir";
for(keys %c){
	my $test="$output_db_dir/test-pics/$_";
	my $train="$output_db_dir/train-pics/$_";
	system "mkdir -p $test";
	system "mkdir -p $train";
	my @cars=@{$c{$_}};
	my $n=scalar @cars;
	my $n_test=int($n*$test_ratio);
	print "n_test=$n_test\n";

	for my $i ( 0 .. $n_test-1 ) {
		print "test: $i/",$n_test-1,"\n";
		system "cp $cars[$i] $test 2>/dev/null";
	}
	for my $i ( $n_test .. $n-1 ) {
		print "train: $i/",$n-1,"\n";
		system "cp $cars[$i] $train 2>/dev/null";
	}
}
my $q;
print "Test Cars:", $q=$total*$test_ratio," Train Cars:", $total-$q," Total: $total\n";
