#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;

use CarTypeRecognition::SIFT::NormalizeGradBlur;
use Getopt::Long;

sub dump_sift($)
{
	my ($s)=@_;
	for(@$s){
		print FH "$_ " for @$_;
	}
}

sub print_im($$)
{
	my ($sift, $i)=@_;
	print FH "    $i: ";	
	my $s=$sift->create_SIFT($i);
	dump_sift($s);
	print FH "\n";
}

my $test_dbs;
my $learn_db;
my $outfile;

my $help=<<EOH;
$0 --learning-database|l <dir> --testing-databases|d <dir1,dir2,...> --outfile|o <file>
EOH
my %Options=(
		'--testing-databases|d=s'   => \$test_dbs,
		'--learning-database|l=s'   => \$learn_db,
		'--outfile|o=s'   => \$outfile,
		);
die $help unless @ARGV;
GetOptions( %Options ) or die "GetOptions failure: $!"; 

open FH, ">$outfile" or die "open($outfile):$!\n";
my $sift=CarTypeRecognition::SIFT::NormalizeGradBlur->new({
	'-grad-bins'  =>  13,
	'-boxes-horiz'=>  15,
	'-boxes-vert' =>  9,
	'-overlap'=>0,
	'-sift-dir'   => "$ENV{HOME}/.car-recognition/sift",
	});
print "\n\n\n";
print FH "---\n";

my @li=`find $learn_db -type f`;
print Dumper \@li;
print FH "- name: $learn_db\n"; 
print FH "  data:\n";
for my $i (@li){
	chomp $i;
	print "I=$i\n";
	print_im($sift, $i);
}
my @test_dbs=split(/ /, $test_dbs);

for my $db (@test_dbs){
	print FH "- name: $db\n"; 
	print FH "  data:\n";
	my @ti=`find $db -type f`;
	for my $i (@ti){
		chomp $i;
		print "I=$i\n";
		print_im($sift, $i);
	}
}

close FH;
