/* example1.c */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "emd.h"

#define min(a,b) (((a)<(b)) ? (a) : (b))
									 
static int N=0;
static int Nsig=0;
float dist(feature_t *F1, feature_t *F2)
{
  int dX = *F1, dY = *F2;
  dY--; dX--;
  int d1=(dX-dY); 
  int d2=(dY-dX); 
  if( d1<0 ) d1+=N;
  if( d2<0 ) d2+=N;
  d1%=N;
  d2%=N;
  int d=min(d1,d2);
  return d*d; 
}


 // printf("dx=%d, dy=%d, d=%d N=%d\n", dX, dY, d, N);

#define fatal(msg...) do { fprintf(stderr,"\nFATAL  : " msg); exit(EXIT_FAILURE); } while(0);
#define info(msg...)       fprintf(stderr,"\nINFO   : " msg)
#define warn(msg...)       fprintf(stderr,"\nWARNING: " msg)
		
static void *xmalloc(void *x, size_t nmem)
{
	void *r=malloc(sizeof(x)*nmem);
	if( r==NULL ) fatal("malloc(%d) failed!", nmem);	
	return r;
}

static void
sanity_check(void)
{
	if( !N )      fatal("No elements found!");
	if( !Nsig )   fatal("No signatures found!");
	if( Nsig<=1 ) fatal("Only one signature found!");
}


static signature_t ** 
read_config( const char *filename )
{
	int i,j;
	FILE *fin;
	float **w;
	feature_t **f;
	signature_t **s;

	if( !filename ) {
		fin=stdin;
		warn("Reading from stdin.");
	}
	else
		if( (fin=fopen(filename,"r"))==NULL ) fatal("Cannot open %s!",filename);

	fscanf(fin, "%d %d\n", &N, &Nsig );
	info("Found number of elements (N) : %d", N);
	info("Found number of signatures (Nsig) : %d", Nsig);

	sanity_check(); /* Check for number of elements N and number of signatures Nsig */

	s=xmalloc(s,Nsig);
	w=xmalloc(w,Nsig);
	f=xmalloc(f,Nsig);
	for(i=0; i<Nsig; i++) {
		s[i]=xmalloc(s[i],1);
		f[i]=xmalloc(f[i],N);
		w[i]=xmalloc(w[i],N);
	}
	info("Signature storage space allocated");
	
	for(j=0; j<Nsig; j++) for(i=0; i<N;i++) 	f[j][i]=i+1;
	info("Features initialized...");

	for(j=0;j<Nsig;j++) {
		for(i=0; i<N;i++) {
			float *p=&w[j][i];
			fscanf(fin, "%f", p);
		}
		fscanf(fin,"\n");
	}
	fclose(fin);

	for(j=0;j<Nsig;j++){
		s[j]->n = N;
		s[j]->Features=f[j];
		s[j]->Weights =w[j];
	}

	return s;
}

static void at_exit(void) { fprintf(stderr,"\n"); }

int main(int argc, char *argv[])
{
	float e, min_e;
	signature_t **s;
	int j, min_j;

	atexit(at_exit);	

	s = read_config(argv[1]);

	min_j=1;
	for(j=1;j<Nsig;j++) {
		e = emd(s[0], s[j], dist, 0, 0);
		if( j>1 &&  min_e < e ) {
			min_e = e;
			min_j = j;
		} 
		if( j==1 ){
			min_j = 1;
			min_e = e;
		}
		printf("emd(s%d,s%d)=%f\n", 0, j, e);
	}

	printf("\nSOLUTION: min_j=\"%d\" min_e=\"%f\"\n", min_j, min_e);
	return(EXIT_SUCCESS);
}
