#!/usr/bin/perl
#
# Perl frontend for LDA classification (done using Octave/Matlab)
# Usage:
# octave `./lda.pl`
use strict;
use warnings;
use Data::Dumper;

# %class = load_file($filename)
sub load_file($)
{
	my($file)=@_;
	local *F;
	open F, "<$file" or die "open($file):$!\n";
	my %class=();
	my $class_label;
	while(<F>){
		chomp;
		s/^\s*//; # skip spaces from the beginning
		s/\s*$//; # skip spaces from the end
		next if /^#/;    # skip comments
		next if /^\s*$/; # skip empty lines
	    if(/^(.*):/){
#			print "GOT: $1\n";
			$class_label=$1;
			$class{$class_label}=[] unless $class{$class_label};
			next;
		}
	
		push @{$class{$class_label}},$_;
	}
	close F;
	return %class;
}

die "Usage:$0 <vectors.learn> <vectors.test>\n" unless @ARGV==2;
my %learn_data=load_file($ARGV[0]);
my %test_data=load_file($ARGV[1]);

#print STDERR Dumper \%learn_data;
#print STDERR Dumper \%test_data;

my @X=();
my $d=0;
my %translated_learn_names=();
my @N=();
local *FLEARN;
my $lda_learn="/tmp/lda-learn-$$.m";
print STDOUT "$lda_learn\n";
open FLEARN, ">$lda_learn" or die "open:$!\n";

for(keys %learn_data){
	print FLEARN "% Class name $_\n";
	print FLEARN "D$d=[\n"; push @X,"D$d";
	$translated_learn_names{$d}=$_;
	print FLEARN "$_\n" for @{$learn_data{$_}};
	push @N, scalar(@{$learn_data{$_}});
	print FLEARN "];\n";
	print FLEARN "% Class $_ mean:\n";
	print FLEARN "m$d=mean(D$d);\n";
	print FLEARN "s$d=(diag(cov(D$d),0))';\n";
	print FLEARN "printf(\"MEAN:\"); printf(\"%f \",m$d);printf(\"\\n\");\n\n";
	print FLEARN "printf(\"VAR :\"); printf(\"%f \",s$d);printf(\"\\n\");\n\n";
	$d++;
}
$d--;

print FLEARN "% Learn set names\n";
print FLEARN "LEARN=[\n";
print FLEARN "\"$translated_learn_names{$_}\"\n" for(0..$d);
print FLEARN "];\n";
print FLEARN "%Class numbers assignment \n";
print FLEARN "y=[\n";
my $i=0;
for my $n (@N){
	$i++;	
	print FLEARN "$i " x $n;
}
print FLEARN "\n];\n\n";

print FLEARN "% Construct input vector X as an assembly of D_i\n";
print FLEARN "X=[\n";
print FLEARN "$_\n" for @X;
print FLEARN "];\n\n";
my $code=<<EOC;
D.X=X';
D.y=y;
model=lda(D,max(y)-1);
%out_data=linproj(D,model);
%figure; ppatterns(out_data,'num');
EOC
print FLEARN $code;
print FLEARN "% project mean and variance to new space\n";
print FLEARN "m$_*=model.W;\n" for (0 .. $d);
print FLEARN "s$_*=model.W;\n" for (0 .. $d);


for my $tc_name (keys %learn_data) {
	for( @{$learn_data{$tc_name}} ) {
		print FLEARN "printf(\"LEARN($tc_name):\");";
		print FLEARN "printf(\"%f \", [$_]*model.W);";
		print FLEARN "printf(\"\\n\");\n";
	}
}
$i=0;
for my $tc_name (keys %test_data){
	my @l=@{$test_data{$tc_name}};
	for my $x (@l){
		$i++;
		print FLEARN "proj_x=[$x]*model.W;\n";
		for(0 .. $d){
			print FLEARN "r$_ = (proj_x - m$_)./s$_;\n";
		}
		print FLEARN "R$i=[\n";
		print FLEARN "r$_*r$_'\n" for (0 .. $d);
		print FLEARN "];\n";
		print FLEARN "[m,i]=min(R$i);\n";
		print FLEARN "printf(\"CLASSIFICATION:%s, %s\\n\",LEARN(i,:), \"$tc_name\");\n";
		print FLEARN "printf(\"TEST($tc_name):\");printf(\"%f \",proj_x); printf(\"\\n\");\n\n";
	}
}


close FLEARN;
