#include <stdio.h>

#include "image.h"
#include "xfunc.h"
#include "classification.h"
#include "sift.h"
#include "error.h"

static db_t database;
static int train_ptr=0;
static int test_ptr=0;

extern char *dir_to_process;
/* reutrns -1 if not found, otherwise index >=0 */
__inline__ int
get_image_picnum(char *filename)
{
	int i;
	for(i=0;i<train_ptr;i++){
		if     (!strcmp(database.train_set[i].i_filename,filename)){
			return database.train_set[i].pic_num;
		}
	}
	for(i=0;i<test_ptr;i++){
		if     (!strcmp(database.test_set[i].i_filename,filename)){
			return database.test_set[i].pic_num;
		}
	}
	return -1;
}


void
set_img_type(char *filename, int pic_num)
{
	subset_t *s;
	if(strstr(filename, "train")){
		if( train_ptr >= MAX_PICTURES ) {
			warn("%s will not go into training subset since MAX_PICTURES exceeded!\n", filename);
			return;
		}
		s=&(database.train_set[train_ptr]);
		train_ptr++;
	}
	else if(strstr(filename, "test")){
		if( test_ptr >= MAX_PICTURES ) {
			warn("%s will not go into test subset since MAX_PICTURES exceeded!\n", filename);
			return;
		}
		s=&(database.test_set[test_ptr]);
		test_ptr++;
	}
	else{
		warn("Cannot determine subset type for image %s!\n",filename);
	}
	s->i_filename=strdup(filename);
	s->pic_num   =pic_num;
}


double *construct_vector(double **sift)
{
	int bins=get_sift_bins();
	int bh  =get_sift_bh();
	int bv  =get_sift_bv();
	double *v=xmalloc(sizeof(double)*bins*bh*bv);
	int i=0, b,t;
	for(t=0;t<bh*bv;t++)
		for(b=0;b<bins;b++){
			v[i++]=sift[t][b];
		}

	return v;
}
extern double h[MAX_PICTURES][SIFT_MAX_TILES][SIFT_MAX_BINS];
void
go_through_sets(void)
{
	int i_test,i_train,i,j; 
	int best=-1;
	double *sift_train, *sift_test,*v_test,*v_train,err,min_err=-1000.0;
	printf("---\n");
	printf("- name: %s-train\n", dir_to_process);
	printf("  data:\n");
	for(i=0;i<train_ptr;i++){
		i_train   =database.train_set[i].pic_num;
		sift_train=&(h[i_train]);
		printf("    %s: ", database.train_set[i].i_filename);
		for(j=0;j<get_sift_bv()*get_sift_bh()*get_sift_bins();j++){
			printf("%.15f ", sift_train[j]);
		}
		printf("\n");
	}

	printf("- name: %s-test\n", dir_to_process);
	printf("  data:\n");
	for(i=0;i<test_ptr;i++){
		i_test    =database.test_set[i].pic_num;
		sift_test =&(h[i_test]);
		printf("    %s: ", database.test_set[i].i_filename);
		for(j=0;j<get_sift_bv()*get_sift_bh()*get_sift_bins();j++){
			printf("%.15f ", sift_test[j]);
		}
		printf("\n");
	}
}
