
/*
 * Guillaume Cottenceau (gc at mandrakesoft.com)
 *
 * Copyright 2002 MandrakeSoft
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <unistd.h>

#include "image.h"
#include "sift.h"
#include "xfunc.h"
#include "files.h"

static char *image_to_process;
char *dir_to_process;
static char *sift_type_str;

void check_cfg(void)
{
	if(get_sift_bins()>SIFT_MAX_BINS){
		fprintf(stderr, "sift_bins(%d)>SIFT_MAX_BINS(%d)!\n", get_sift_bins(), SIFT_MAX_BINS);
		exit(-1);
	}
	if(get_sift_bv()*get_sift_bh()>SIFT_MAX_TILES){
		fprintf(stderr, "SIFT_MAX_TILES(%d) exceeded by your SIFT topology!\n", SIFT_MAX_TILES);
		exit(-1);
	}
	if(!sift_type_str){
		fprintf(stderr, "Do not know which sift type to use!\n");
		exit(-1);
	}
	if(!strcmp(sift_type_str,"simple")){
		set_sift_type(SIFT_SIMPLE);
	}
	else if(!strcmp(sift_type_str,"grad")){
		set_sift_type(SIFT_GRAD);
	}
	else if(!strcmp(sift_type_str,"grad-blur")){
		set_sift_type(SIFT_GRAD_BLUR);
	}
	else {
		fprintf(stderr, "sift type must be one of: ``simple'', ``grad'', ``grad-blur''!\n");
		exit(-1);
	}
}

static void
read_cfg(int argc, char *argv[])
{
opterr = 0;
int c;
     
while ((c = getopt (argc, argv, "h:v:b:i:d:t:o")) != -1)
	switch (c){
		case 'h':
			set_sift_bh(atoi(optarg));
			break;
		case 'v':
			set_sift_bv(atoi(optarg));
			break;
		case 'b':
			set_sift_bins(atoi(optarg));
			break;
		case 'i':
			image_to_process=optarg;	
			break;
		case 'd':
			dir_to_process=optarg;
			break;	
		case 't':
			sift_type_str=optarg;
			break;	
		case 'o':
			set_sift_overlap(1);
		case '?':
			if (isprint (optopt))
				fprintf (stderr, "Unknown option `-%c'.\n", optopt);
			else
				fprintf (stderr,
						"Unknown option character `\\x%x'.\n",
						optopt);
			return ;
		default:
			abort ();
	}
}

static void
print_cfg()
{
	fprintf(stderr,"\nSift Type:");
	switch(get_sift_type()){
		case SIFT_SIMPLE: fprintf(stderr,"simple"); break;
		case SIFT_GRAD_BLUR: fprintf(stderr,"grad-blur"); break;
		case SIFT_GRAD: fprintf(stderr,"grad"); break;
	}
	fprintf(stderr,"\nOverlap: %d\n", get_sift_overlap());
	fprintf(stderr,"Topolgy: %dx%dx%d\n", get_sift_bh(), get_sift_bv(), get_sift_bins());
}

int main(int argc, char **argv)
{
	int num_of_files;
	char **files;
	db_files_t *db_files;
	read_cfg(argc, argv);
	check_cfg();
	print_cfg();
	if(image_to_process) {
		num_of_files=1;
		files=xmalloc(sizeof(char*)*num_of_files);
		files[0]=strdup(image_to_process);
	}
	else if(dir_to_process){
		db_files=get_database_files(dir_to_process);
		num_of_files=db_files->num_of_files;
		files       =db_files->db;
	}
	else {
		fprintf(stderr, "No dir or image specified, nothing to do!\n");
		exit(-1);
	}
	read_images(files, num_of_files);
	process_images(num_of_files);
	go_through_sets();
	//print_sifts(num_of_files);
	//write_png_file(argv[2]);

	return 0;
}


