
/*
 * Guillaume Cottenceau (gc at mandrakesoft.com)
 *
 * Copyright 2002 MandrakeSoft
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * 2005-1-12 Changed by Michal Conos
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <unistd.h>

#define PNG_DEBUG 3
#include <png.h>

#include "image.h"
#include "sift.h"

static int sift_type=SIFT_GRAD_BLUR;
static int sift_bh=15;
static int sift_bv=9;
static int sift_bins=13;
static int sift_overlap=0;


double h[MAX_PICTURES][SIFT_MAX_TILES][SIFT_MAX_BINS]; /* 1024*30*500*8 = 117MB*/

void set_sift_type(int stype)
{
	sift_type=stype;
}

int get_sift_type(void)
{
	return sift_type;
}



void set_sift_bh(int bh)
{
	sift_bh=bh;
}

int get_sift_bh(void)
{
	return sift_bh;
}

void set_sift_bv(int bv)
{
	sift_bv=bv;
}

int get_sift_bv(void)
{
	return sift_bv;
}

void set_sift_bins(int bins)
{
	sift_bins=bins;
}

int get_sift_bins(void)
{
	return sift_bins;
}

void set_sift_overlap(int overlap)
{
	sift_overlap=overlap;
}

int get_sift_overlap(void)
{
	return sift_overlap;
}

void
process_images(int n)
{
	int pic_num;
	for(pic_num=0;pic_num<n;pic_num++){
		fprintf(stderr, "Processing images: %.2f%% done.\r", (double)100*pic_num/n); fflush(stderr);
		process_picture(pic_num);
	}
	fprintf(stderr, "\n");
}

static __inline__ 
void compute_derivation(double x, double y, int pic_num, int *a, double *g)
{
	double dx, dy;

	dy =  Iv( x-1, y-1,pic_num ) + 2*Iv(  x, y-1,pic_num ) + Iv( x+1, y-1,pic_num ) -
	    ( Iv( x-1, y+1,pic_num ) + 2*Iv(  x, y+1,pic_num ) + Iv( x+1, y+1,pic_num ) );

	dx =  Iv( x+1, y-1,pic_num ) + 2*Iv( x+1, y,pic_num ) + Iv( x+1, y+1,pic_num ) -
	    ( Iv( x-1, y-1,pic_num ) + 2*Iv( x-1, y,pic_num ) + Iv( x-1, y+1,pic_num ) );

	*a = (int)floor(0.5+atan2(dy, dx)*180/M_PI);
	*a+=180;
	*a%=180;

	dx/=256;
	dy/=256;

	*g=sqrt(dx*dx+dy*dy);
}

static __inline__ void 
process_tile(double sx, double sy, double stw, double sth, int tile_num, int pic_num)
{
	int a,b;
	double g;
	double cx=sx+(stw/2);
	double cy=sy+(sth/2);
	double x,y;
	double grad_sum=0;

//	fprintf(stderr, "sy=%.2f sth=%.2f\n", sy, sth);
	for(y=sy;y<sy+sth-1;y++){
		double dy=cy-y;
		for(x=sx;x<sx+stw-2;x++){
			double dx=cx-x;
			double d =sqrt(dx*dx+dy*dy);
			if(d==0)d=1;
			compute_derivation(x,y,pic_num, &a, &g);
			//              rgba_t *rgba=get_RGBA(x,y,pixel_width);
			//				rgba->r=rgba->b=rgba->g=(int)a*g;
			//				set_RGBA( x, y, rgba, pixel_width );
			a/=180/sift_bins;
			switch(sift_type){
				case SIFT_SIMPLE:
					h[pic_num][tile_num][a]++;
					grad_sum++;
					break;
				case SIFT_GRAD:
					h[pic_num][tile_num][a]+=g;
					grad_sum+=g;
					break;
				case SIFT_GRAD_BLUR:
					h[pic_num][tile_num][a]+=g/d;
					grad_sum+=g/d;
					break;
			}
		}
	}

	/* make a probability distribution */
	for(b=0;b<sift_bins;b++){
		h[pic_num][tile_num][b]/=grad_sum;
	}	
}

void 
process_picture(int pic_num)
{
int tile_num=0;
double stw, sth;
double x,y;
int width=get_image_width(pic_num);
int height=get_image_height(pic_num);
	
	stw=(double)(width-2)/sift_bh;
	sth=(double)(height-2)/sift_bv;

//	fprintf(stderr, "ow=%.2f, oh=%.2f\n", stw,sth);
//	fprintf(stderr, "Width=%d Height=%d\n", width, height);
	
	for (x=1; x<width-stw; x+=stw) {
		for (y=1; y<height-sth; y+=sth) {
//			fprintf(stderr, "***x=%.2f y=%.2f tile_num=%d\n", x,y,tile_num);
			process_tile(x,y,stw,sth,tile_num, pic_num);
			tile_num++;
		}
	}

	if( get_sift_overlap() ){
		for( x=stw/2; x<width-stw/2; x+=stw ){
			for( y=sth/2; y<height-sth/2; y+=sth ) {
				process_tile(x,y,stw,sth,tile_num, pic_num);
				tile_num++;
			}
		}
	}
}


void 
print_sifts(int n)
{
	int i,t,b;
	FILE *fp=stdout;
	for(i=0;i<n;i++){
		fprintf(fp,"my $VAR1=[\n");
		for(t=0;t<sift_bv*sift_bh;t++) {
			fprintf(fp," [\n");
			for(b=0;b<sift_bins;b++) {
				fprintf(fp,"  %f,\n", h[i][t][b]);
			}
			fprintf(fp," ],\n");
		}
		fprintf(fp,"];\n");
	}
}
